/* Copyright (c) 1996, 1997 Prominence Dot Com, Inc.        *
 * See the file legal.txt in the txt directory for details. */

package prominence.msg;

import java.io.*;
import java.util.*;

/**
 * A class that reads messages from a <tt>RoutingInputStream</tt>
 * and forwards them on the <tt>MessageOutput</tt>s specified in the
 * message routing header.
 *
 * @version 1.0 1 Nov 1996
 * @author Merlin Hughes
 * @see prominence.msg.Demultiplexer
 */
public class Router extends Thread {
  /**
   * The <tt>RoutingInputStream</tt> from which messages are read.
   */
  protected RoutingInputStream i;
  /**
   * The message routing table. Maps from routing names to
   * <tt>MessageOutput</tt>s.
   */
  protected Hashtable routes;

  /**
   * Current <tt>Router</tt> ID.
   */
  static private int routerNumber;
  /**
   * Assigns unique <tt>Router</tt> ID's.
   * @return An unique <tt>Router</tt> ID.
   */
  static private synchronized int nextRouterNum () { return routerNumber ++; }

  /**
   * Creates a new <tt>Router</tt> reading from a specified stream.
   * @param i The <tt>RoutingInputStream</tt> from which messages should be read
   */
  public Router (RoutingInputStream i) {
    super ("Router-" + nextRouterNum ());
    this.i = i;
    routes = new Hashtable ();
  }

  /**
   * Registers a <tt>MessageOutput</tt> as the destination for messages to
   * a particular target.
   * @param client The client name that is to be routed
   * @param o The destination for such messages
   */
  public void register (String client, MessageOutput o) {
    routes.put (client, o);
  }

  /**
   * Deregisters a particular target name.
   * @param client The client name that is to be deregistered
   */
  public void deregister (String client) {
    routes.remove (client);
  }

  /**
   * Routes messages from the <tt>RoutingInputStream</tt> to the
   * <tt>MessageOutput</tt>s identified by their routing headers.
   * <p>This method is called by a new thread when the superclass
   * <tt>start()</tt> method is called.
   * @see java.lang.Thread#start
   */
  public void run () {
    try {
      while (true) {
        i.receive ();
        byte[] buffer = new byte[i.available ()];
        i.readFully (buffer);
        if (i.target == null)
          broadcast (buffer);
        else
          multicast (i.target, buffer);
      }
    } catch (IOException ex) {
      ex.printStackTrace ();
    }
  }

  /**
   * Broadcasts a message to all streams registered in the routing
   * table.
   * @param buffer The message to be broadcast
   */
  protected void broadcast (byte[] buffer) {
    Enumeration e = routes.elements ();
    while (e.hasMoreElements ()) {
      MessageOutput o = null;
      synchronized (routes) {
        if (e.hasMoreElements ())
          o = (MessageOutput) e.nextElement ();
      }
      if (o != null) {
        try {
          synchronized (o) {
            o.write (buffer);
            o.send ();
          }
        } catch (IOException ex) {
          ex.printStackTrace ();
        }
      }
    }
  }

  /**
   * Broadcasts a message to a specified list of recipients.
   * @param dst a List of recipient names
   * @param buffer The message to be broadcast
   */
  protected void multicast (String[] dst, byte[] buffer) {
    for (int j = 0; j < dst.length; ++ j) {
      MessageOutput o = (MessageOutput) routes.get (dst[j]);
      if (o != null) {
        try {
          synchronized (o) {
            o.write (buffer);
            o.send ();
          }
        } catch (IOException ex) {
          ex.printStackTrace ();
        }
      }
    }
  }
}