/* Copyright (c) 1996, 1997 Prominence Dot Com, Inc.        *
 * See the file legal.txt in the txt directory for details. */

package prominence.msg;

import java.io.*;

/**
 * A <tt>MessageOutput</tt> that attaches to an existing <tt>MessageOutput</tt>
 * and attaches a routing list to the header of each message that is
 * transmitted.
 * <p>The routing list is specified with the targeted <tt>send()</tt>
 * methods.
 *
 * @version 1.0 1 Nov 1996
 * @author Merlin Hughes
 * @see prominence.msg.RoutingInputStream
 */
public class RoutingOutputStream extends MessageOutput {
  /**
   * The <tt>MessageOutput</tt> to which this is attached.
   */
  protected MessageOutput o;
  /**
   * A <tt>ByteArrayOutputStream</tt> used to buffere the current message
   * contents.
   */
  protected ByteArrayOutputStream byteO;

  /**
   * Creates a new <tt>RoutingOutputStream</tt>.
   * @param o The <tt>MessageOutput</tt> to which to send messages
   */
  public RoutingOutputStream (MessageOutput o) {
    super (new ByteArrayOutputStream ());
    byteO = (ByteArrayOutputStream) out;
    this.o = o;
  }

  /**
   * Sends the current message with a <tt>null</tt> routing list; this
   * is usually interpreted as a broadcast.
   * @exception IOException Occurs if there is a problem sending the
   * message.
   */
  public void send () throws IOException {
    synchronized (o) {
      o.writeInt (-1);
      byteO.writeTo (o); 
      o.send ();
    }
    byteO.reset ();
  }

  /**
   * Sends the current message with the specified routing list. The
   * call is passed on as an untargeted <tt>send()</tt> to the
   * attached stream.
   * @exception IOException Occurs if there is a problem sending the
   * message.
   */
  public void send (String[] dst) throws IOException {
    if (dst == null)
      send ();
    else {
      synchronized (o) {
        o.writeInt (dst.length);
        for (int i = 0; i < dst.length; ++ i)
          o.writeUTF (dst[i]);
        byteO.writeTo (o); 
        o.send ();
      }
      byteO.reset ();
    }
  }

}